#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Predesys - Predesys Server
# Version: 0.1.0
# Author: Jose Antonio Jimenez Carmona <josantjim@gmail.com>
# License: LGPL 3.0 (see "license" file)

import sys
sys.dont_write_bytecode = True
import time
import coreexceptions
import sqlalchemy
import data
import configuration

# Row limit
ROW_LIMIT = 1000

def get_results_limit():
    global ROW_LIMIT
    return ROW_LIMIT

def get_log_addresses(limit):
    global ROW_LIMIT   
    if limit == None or type(limit) != int or limit < 1 or limit > ROW_LIMIT: limit = ROW_LIMIT
    
    addresses = []
    
    try:
        dbConnection = data.open_connection()
        statement = "select * from address_log order by log_datetime desc limit %d;" % limit
        result = data.execute(dbConnection, statement)
        for row in result:
            addresses.append([row["address"], row["station_user_id"], float(row["log_datetime"])])
        result.close()
        data.close_connection(dbConnection)
    except:
        raise
        
    return addresses

def get_log_registered_addresses(limit):
    global ROW_LIMIT
    if limit == None or type(limit) != int or limit < 1 or limit > ROW_LIMIT: limit = ROW_LIMIT
    addresses = []
    
    try:
        dbConnection = data.open_connection()
        statement = "select address_log.address, station_user_id, log_datetime from address_log, addresses where address_log.address = addresses.address order by log_datetime desc limit %d;" % limit
        result = data.execute(dbConnection, statement)
        for row in result:
            addresses.append([row["address"], row["station_user_id"], float(row["log_datetime"])])
        result.close()
        data.close_connection(dbConnection)
    except:
        raise
        
    return addresses

def get_last_log_modified_date():
    date = None
    
    try:
        dbConnection = data.open_connection()
        statement = "select last_mod_date from address_log_dates;"
        result = data.execute(dbConnection, statement)
        row = result.fetchone()
        if row: date = float(row["last_mod_date"])
        else: date = time.time()
        result.close()
        data.close_connection(dbConnection)
    except:
        raise
        
    return date
    
def update_dates(updateTime):
    if updateTime == None or (type(updateTime) != int and type(updateTime) != long and type(updateTime) != float): raise coreexceptions.SystemException("'updateTime' is not an integer or a long integer or a float")
        
    try:
        dbConnection = data.open_connection()
        statement = "delete from address_log_dates;"
        result = data.execute(dbConnection, statement)
        result.close()
        statement = "insert into address_log_dates values ('%s');" % updateTime
        result = data.execute(dbConnection, statement)
        result.close()
        data.close_connection(dbConnection)
    except:
        raise
    
def reset_log(updateTime):
    if updateTime == None or (type(updateTime) != int and type(updateTime) != long and type(updateTime) != float): raise coreexceptions.SystemException("'updateTime' is not an integer or a long integer or a float")
    
    try:
        dbConnection = data.open_connection()
        statement = "delete from address_log;"
        result = data.execute(dbConnection, statement)
        result.close()      
        data.close_connection(dbConnection)  
        update_dates(updateTime)
    except:
        raise
        
def do_reset():
    doReset = False
    
    try:
        dbConnection = data.open_connection()
        statement = "select count(*) as c from address_log;"
        result = data.execute(dbConnection, statement)
        row = result.fetchone()
        doReset = (row and int(row["c"]) >= 10000)
        result.close()
        data.close_connection(dbConnection)
    except:
        raise
        
    return doReset
    
def update_log_addresses(addresses, stationUserId):
    if addresses != None and type(addresses) != list:
        raise coreexceptions.SystemException("'addresses' is not a string list")
    if stationUserId == None or (type(stationUserId) != str and type(stationUserId) != unicode) or stationUserId == "":
        raise coreexceptions.ValueNotValidException("'stationUserId' must be a not null and not empty string")

    try:
        updateTime = time.time()

        if do_reset(): 
            reset_log(updateTime)
            logReset = True
        else:
            logReset = False

        if addresses != None and len(addresses) > 0:    
            dbConnection = data.open_connection()      
            for addr in addresses:
                statement = "insert into address_log values ('%s', '%s', '%s');" % (addr, stationUserId, updateTime)
                result = data.execute(dbConnection, statement)
                result.close()      
            data.close_connection(dbConnection)
            if not logReset: update_dates(updateTime)
    except:
        raise
