#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Predesys - Predesys Server
# Version: 0.1.0
# Author: Jose Antonio Jimenez Carmona <josantjim@gmail.com>
# License: LGPL 3.0 (see "license" file)

import sys
sys.dont_write_bytecode = True
import os
import os.path
DIR = os.path.abspath(os.path.dirname(os.path.realpath(__file__)))
import exceptions
import coreexceptions
import sqlalchemy
import sqlalchemy.engine.base
import configuration
    
def get_supported_db_systems():
    return ["mysql"]

def open_connection(): 
    connection = None
    config = configuration.load_configuration()

    dbType = config.get_db_type()
    if dbType == None: raise coreexceptions.DataBaseException("Data base settings not set") 
    
    dbHost = config.get_db_host()
    if dbHost == None: raise coreexceptions.DataBaseException("Data base settings not set") 
    
    dbPort = config.get_db_port()
    
    dbName = config.get_db_name()
    if dbName == None: raise coreexceptions.DataBaseException("Data base settings not set") 
    
    dbUserName = config.get_db_user_name()
    if dbUserName == None: raise coreexceptions.DataBaseException("Data base settings not set") 
    
    dbUserPassword = config.get_db_user_password()
    if dbUserPassword == None: raise coreexceptions.DataBaseException("Data base settings not set") 
    
    if dbType == "mysql":
        if dbPort == None: dbConnectionString = "mysql+mysqldb://%s:%s@%s/%s" % (dbUserName, dbUserPassword, dbHost, dbName)
        else: dbConnectionString = "mysql+mysqldb%s://%s:%s@%s:%d/%s" % (dbUserName, dbUserPassword, dbHost, dbPort, dbName)     
        
        try: 
            connection = (sqlalchemy.create_engine(dbConnectionString, encoding="utf8")).connect()
        except:
            raise coreexceptions.DataBaseException("Data base connection error")
    else:
        raise coreexceptions.DataBaseException("Data base type not supported")
        
    return connection
    
def execute(connection, statement):
    if connection == None or type(connection) != sqlalchemy.engine.base.Connection: raise coreexceptions.ValueNotValidException("'connection' must be a not null Connection object.")
    if statement == None or (type(statement) != str and type(statement) != unicode) or statement == "": raise coreexceptions.ValueNotValidException("'statement' must be a valid SQL statement.")
    filter_sql(statement)
    
    try:
        return connection.execute(statement)
    except:
        raise coreexceptions.DataBaseException("Data base error")
        
def filter_sql(sentence):
    # This function must prevent database structure modification and throw exceptions if needed
    pass

def close_connection(connection):
    if connection == None or type(connection) != sqlalchemy.engine.base.Connection: raise coreexceptions.ValueNotValidException("'connection' must be a not null Connection object.")
    
    try:
        connection.close()
    except:
        raise coreexceptions.DataBaseException("Data base closing error")
