#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Predesys - Predesys Server
# Version: 0.1.0
# Author: Jose Antonio Jimenez Carmona <josantjim@gmail.com>
# License: LGPL 3.0 (see "license" file)

import sys
sys.dont_write_bytecode = True
import os
import os.path
import commands
import time
from signal import SIGTERM
import daemon
from SimpleXMLRPCServer import SimpleXMLRPCServer
from SimpleXMLRPCServer import SimpleXMLRPCRequestHandler
DIR = os.path.abspath(os.path.dirname(os.path.realpath(__file__)))
sys.path.append(os.path.join(DIR, "../core"))
import configuration
import services

# File to save process ID
DAEMON_PID_FILE = "/tmp/predesys-server-webservice.pid"
    
# Service method
def get_services():
    return services.get_enabled_services_by_type("user-callable")
    
# Service method
def get_service_description(serviceId):
    if serviceId == None or (type(serviceId) != str and type(serviceId) != unicode) or serviceId == "": raise Exception("Service ID must be a not null and not empty string")
    if not services.is_service(serviceId): raise Exception("Service ID not valid")
    
    serviceData = services.get_service_data(serviceId)
    if serviceData.get_type().get_id() != "user-callable": raise Exception("Service ID not valid")
    
    return "<description><name>%s</name><authenticated>%d</authenticated></description>" % (serviceData.get_name(), int(serviceData.is_authenticated()))

# Service method
def run_service(serviceId, args):
    if serviceId == None or (type(serviceId) != str and type(serviceId) != unicode) or serviceId == "": raise Exception("Service ID must be a not null and not empty string")
    if args != None and type(args) != list: raise Exception("Service arguments must be null or a string array")
    if not services.is_service(serviceId): raise Exception("Service ID not valid")
    
    serviceData = services.get_service_data(serviceId)
    if serviceData.get_type().get_id() != "user-callable": raise Exception("Service ID not valid")
    
    result = None

    try:
        result = services.run_service(serviceId, args)
    except:
        raise Exception("Error running service")
    
    if result == None: result = ""
    return result
    
# Service method
def run_authenticated_service(serviceId, args, userName, userPassword):
    if serviceId == None or (type(serviceId) != str and type(serviceId) != unicode) or serviceId == "": raise Exception("Service ID must be a not null and not empty string")
    if args != None and type(args) != list: raise Exception("Service arguments must be null or a string array")
    if userName == None or (type(userName) != str and type(userName) != unicode) or userName == "": raise Exception("User name must be a not null and not empty string")
    if userPassword == None or (type(userPassword) != str and type(userPassword) != unicode) or userPassword == "": raise Exception("User password must be a not null and not empty string")
    if not services.is_service(serviceId): raise Exception("Service ID not valid")
    
    serviceData = services.get_service_data(serviceId)
    if serviceData.get_type().get_id() == "user-auto": raise Exception("Service ID not valid")
    
    result = None

    try:
        result = services.run_auth_service(serviceId, userName, userPassword, args)
    except:
        raise Exception("Error running service")
     
    if result == None: result = ""
    return result
    
class PredesysServerWebServiceProxy(daemon.Daemon):
    """Predesys Server Web Service daemon.
    
    It does all the work of the proxy. 
    """
    def __init__(self):
        daemon.Daemon.__init__(self, pidfile=DAEMON_PID_FILE)
              
    def run(self):
        """Does the daemon work.
        
        It keeps running the web service.
        """
        self.__load_configuration()
                
        server = SimpleXMLRPCServer((self.config.get_webservice_host(), self.config.get_webservice_port()))
        server.register_function(get_services)
        server.register_function(get_service_description)
        server.register_function(run_service)
        server.register_function(run_authenticated_service)

        # Run the server's main loop
        server.serve_forever()
        
    def __load_configuration(self):
        """Loads configuration from CONFIG_FILE_1 file or CONFIG_FILE_2 file.
        
        If an error occurs, it sets the default configuration.
        """
        self.config = configuration.load_configuration()
        
if __name__ == "__main__":
    if len(sys.argv) == 2:
        if sys.argv[1] == "start":
            PredesysServerWebServiceProxy().start()
        elif sys.argv[1] == "restart":
            PredesysServerWebServiceProxy().restart()
        elif sys.argv[1] == "stop":
            PredesysServerWebServiceProxy().stop()
        else:
            print "Unknown command"
            sys.exit(2)
        sys.exit(0)
    else:
        print "Usage: %s start|stop|restart" % sys.argv[0]
        sys.exit(2)
